/* Copyright 2010 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: App.h 230 2010-12-18 14:22:31Z tonic $
 * $Revision: 230 $
 */

#ifndef APP_H_INCL
#define APP_H_INCL


#include <assert.h>


class AppState;


class App : public IEventReceiver
{
public:

    App(irr::IrrlichtDevice *irrDevice);
    virtual ~App();

    // From IEventReceiver:
    virtual bool OnEvent(const SEvent &event);

    // App:
    static void setInstance(App *singleton);
    static App * getInstance()
    {
        assert(mSingleton != NULL);
        return mSingleton;
    }

    virtual bool init(irr::IrrlichtDevice *irrDevice);

    virtual void run();

    virtual void render() {}

    virtual void setAppStateHandler(AppState *state);
    AppState * getCurrentAppStateHandler() { return mCurrentAppStateHandler; }

    void setAlive(bool alive) { mAlive = alive; }
    bool isAlive() const { return mAlive; }

    UI32 getTime() { return mTime; }

    Graphics * getGraphics() { return mGraphics; }

    virtual wchar_t * getWindowCaption();

    void setMaxLogicUpdatesPerFrame(int updateCap) { mMaxLogicUpdatesPerFrame = updateCap; }

    // Irr-specific:

    I32 getScreenWidth() { return mIrrVideoDriver->getScreenSize().Width; }
    I32 getScreenHeight() { return mIrrVideoDriver->getScreenSize().Height; }

    irr::IrrlichtDevice * getIrrDevice() { return mIrrDevice; }
    irr::video::IVideoDriver * getIrrVideoDriver() { return mIrrVideoDriver; }
    irr::scene::ISceneManager * getIrrSceneManager() { return mIrrSceneManager; }

protected:

    virtual bool update();

    void resetTime();

    static App *mSingleton;

    bool mAlive;
    I32 mTime;
    UI32 mStartTime;
    double mNextUpdateTime;
    I32 mMaxLogicUpdatesPerFrame;

    AppState *mCurrentAppStateHandler;

    I32 mMouseX, mMouseY;
    I32 mPreviousMouseX, mPreviousMouseY;

    Graphics *mGraphics;

    irr::IrrlichtDevice *mIrrDevice;
    irr::video::IVideoDriver *mIrrVideoDriver;
    irr::scene::ISceneManager *mIrrSceneManager;
};


#endif // !APP_H_INCL
