/* Copyright 2010 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: MainState.h 241 2010-12-20 01:59:19Z tonic $
 * $Revision: 241 $
 */

#ifndef MAINSTATE_H_INCL
#define MAINSTATE_H_INCL


#include <vector>

//using namespace std;
using namespace irr;
using namespace scene;
using namespace core;
using namespace video;


enum TILE_TYPE
{
    TT_OPEN_AIR = 0,
    TT_IMPASSABLE,
    TT_GROUND,
    TT_TUNNEL,
    TT_SHOP_DRILL,
    TT_SHOP_SCANBAT,
    TT_COUNT //
};

enum PLAYER_STATE
{
    PLR_MOVING = 0,
    PLR_DIGGING_PICKAXE,
    PLR_DIGGING_DRILL,
    PLR_SCANNING
};

enum PLAYER_ITEM
{
    ITEM_PICKAXE = 0,
    ITEM_DRILL,
    ITEM_SCANNER,
    ITEM_COUNT //
};

enum DIRECTION
{
    DIR_DOWN = 0,
    DIR_UP,
    DIR_LEFT,
    DIR_RIGHT
};

enum SPRITES
{
    SPR_PICKAXE = 8,
    SPR_DRILL = 9,
    SPR_SCANNER = 10
};

struct Tile
{
    Tile() : type(TT_GROUND), x(0.5f), y(0.5f), condition(1), conditionTarget(1), hasGold(false), scannerInfluence(false) {}
    int type;
    float x, y; // [0..1] coords inside tile for a type-specific thing
    float condition, conditionTarget;
    bool hasGold;
    bool scannerInfluence;
};

struct Player
{
    Player() { reset(); }
    void reset()
    {
        state = PLR_MOVING;
        stateTime = 0;
        x = 1.5f; y = 2.5f;
        drillUses = 0;
        scannerUses = 3;
        selItem = 0;
        scannerAngle = (float)(PI / 2);
        scannerDist = 0.5f;
        dir = DIR_DOWN;
        walkTime = 0;
        diamondsFound = 0;
    }
    int state, stateTime;
    float x, y;
    int drillUses, scannerUses;
    int selItem;
    float scannerAngle; // 0..2pi
    float scannerDist; // 0..1
    int dir;
    int walkTime;
    int diamondsFound;
};


class MainState : public AppState
{
public:

    MainState();
    virtual ~MainState();

    // From AppState:

    virtual bool init();
    virtual void deinit();

    virtual bool update(I32 deltaTime);
    virtual void render(irr::IrrlichtDevice *device);

    virtual void onActivate(UI32 time);
    virtual void onDeactivate() {}

    virtual bool onKeyDown(const irr::SEvent::SKeyInput &event);
    virtual bool onKeyUp(const irr::SEvent::SKeyInput &event);
    virtual bool onMouseDown(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseUp(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseMotion(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseWheel(const irr::SEvent::SMouseInput &event);
    // Irr is lacking focus lost/gain events..
    //virtual void onFocusLost(const .. &event) {}
    //virtual void onFocusGain(const .. &event) {}

protected:

    void startGame();

    void renderGame(irr::IrrlichtDevice *device);

    void drawSprite(float x, float y, int num, bool horizFlip = false);
    void genCone(float x, float y, float z, float r, const video::SColor &color);
    void genArc(float x, float y, float r, float uvr, float a1, float a2, const video::SColor &color);

    enum IMAGE_ID
    {
        IMG_BUTTON = 0,
        IMG_BUTTON_HOT,
        IMG_BUTTON_ACTIVE,
        IMG_BUTTON_HOT_ACTIVE,
        IMG_SPRITES,
        IMG_CAVE,
        IMG_SCANNER
    };

    App *mApp;

    IMGUI mIMGUI;
    IMGUIImageButtonStyle *mButtonStyle;

    Font *mFont;
    std::vector<Image *> mImages;
    C8 *mLabelText;

    IrrlichtDevice *mDevice;
    IVideoDriver *mVideo;

    vector2di mLastMousePos;

    bool mTitleUIActive;
    bool mInGame;
    bool mEndingUIActive;

    bool mKeyUp, mKeyDown, mKeyLeft, mKeyRight;

    I32 mScore;

    I32 mLevelWidth, mLevelHeight;
    Tile *mWorld;
    Player mPlayer;

    irr::core::array<irr::u16> mIndices;
    irr::core::array<irr::video::S3DVertex> mVertices;

    int mScanResult;
    float mScanResultTransition;
};


#endif // !MAINSTATE_H_INCL
